@@ -10,6 +10,7 @@ import shlex
 import signal
 import stat
 import sys
+import urllib
 from datetime import datetime, timedelta
 from pathlib import Path
 from typing import TYPE_CHECKING
@@ -23,16 +24,15 @@ from eumdac import DataStore, DataTailor
 from eumdac.collection import SearchResults
 from eumdac.config import get_config_dir, get_credentials_path
 from eumdac.download_app import DownloadApp
-from eumdac.errors import EumdacError
+from eumdac.errors import EumdacError, eumdac_raise_for_status
 from eumdac.fake import FakeDataStore, FakeDataTailor  # type: ignore
 from eumdac.logging import gen_table_printer, init_logger, logger
 from eumdac.order import Order, all_order_filenames, resolve_order
-from eumdac.product import Product
+from eumdac.product import Product, ProductError
 from eumdac.tailor_app import TailorApp
 from eumdac.tailor_models import Chain
 from eumdac.token import AccessToken, AnonymousAccessToken
-
-from .__version__ import __documentation__, __title__, __version__  # noqa
+import eumdac.common
 
 if TYPE_CHECKING:  # pragma: no cover
     from typing import Any, Callable, Collection, Dict, Iterator, Optional, Tuple, Union
@@ -150,7 +150,7 @@ class ProductIterables:
         raise NotImplementedError()
 
 
-def _search(args: argparse.Namespace) -> Collection[Product]:
+def _search(args: argparse.Namespace) -> Tuple[Collection[Product], int]:
     """given search query arguments will return the list of matching products"""
     datastore = get_datastore(args, anonymous_allowed=True)
     query_results = []
@@ -163,6 +163,16 @@ def _search(args: argparse.Namespace) -> Collection[Product]:
     if args.time_range:
         start, end = args.time_range
 
+    # See https://docs.opengeospatial.org/is/13-026r9/13-026r9.html#20 for the mathematical notation expected by the publication filter
+    if args.publication_after and args.publication_before:
+        publication = f"[{args.publication_after.isoformat(timespec='milliseconds')}Z,{args.publication_before.isoformat(timespec='milliseconds')}Z]"
+    elif args.publication_after:
+        publication = f"[{args.publication_after.isoformat(timespec='milliseconds')}Z"
+    elif args.publication_before:
+        publication = f"{args.publication_before.isoformat(timespec='milliseconds')}Z]"
+    else:
+        publication = None
+
     sort_query = None
     if args.sort:
         if args.sort == "ingestion":
@@ -181,6 +191,7 @@ def _search(args: argparse.Namespace) -> Collection[Product]:
     _query = {
         "dtstart": start,
         "dtend": end,
+        "publication": publication,
         "bbox": args.bbox,
         "geo": args.geo,
         "sat": args.sat,
@@ -197,38 +208,43 @@ def _search(args: argparse.Namespace) -> Collection[Product]:
     if bbox is not None:
         query["bbox"] = ",".join(map(str, bbox))
 
+    products: Collection[Product] = []
+    num_products: int = 0
     for collection_id in args.collection:
-        collection = datastore.get_collection(collection_id)
-        query_results.append(collection.search(**query))
-
-    return ProductIterables(query_results, args.limit, query)
+        try:
+            collection = datastore.get_collection(collection_id)
+            query_results.append(collection.search(**query))
+            products = ProductIterables(query_results, args.limit, query)
+            # Check the number of products to execute the search
+            num_products = len(products)
+        except Exception as err:
+            datastore.check_collection_id(collection_id)
+            raise
+    return products, num_products
 
 
 def search(args: argparse.Namespace) -> None:
     """eumdac search entrypoint"""
+
     if args.time_range and (args.dtstart or args.dtend):
         raise ValueError("You can't combine --time-range and --start/--end.")
 
-    products_query = _search(args)
-    products_count = len(products_query)
+    products_query, products_count = _search(args)
 
-    limit = args.limit or 10
+    limit = args.limit or 10000
     products = itertools.islice(products_query, limit)
     if products_count < 1:
         logger.error(f"No products were found for the given search parameters")
         return
-
     if products_count > limit:
-        # show a warning through stderr only when more than 10
+        # show a warning through stderr only when more than 10000
         # products would be shown and limit keyword is not used.
-        logger.warning(f"By default, only 10 of {products_count} products are displayed.")
+        logger.warning(f"By default, only 10000 of {products_count} products are displayed.")
         logger.warning("Please use --limit to increase the number of products if necessary.")
-
     if products_count > 10000:
         logger.error(
             "Notice: EUMETSATs DataStore APIs allow a maximum of 10.000 items in a single request. If more than 10.000 items are needed, please split your requests."
         )
-
     for product in products:
         CRLF = "\r\n"
         logger.info(str(product).replace(CRLF, "-"))
@@ -253,23 +269,22 @@ class AngrySigIntHandler:
         )
 
 
-def safe_run(app: Any, force: bool = False) -> None:
+def safe_run(
+    app: Any, collection: Optional[str] = None, num_products: int = -1, force: bool = False
+) -> None:
     """wrapper around app.run() for exception handling and logging"""
 
-    num_products = len(list(app.order.iter_product_info()))
-    plural = "" if num_products == 1 else "s"
-    logger.info(f"Processing {num_products} product{plural}.")
+    if num_products < 0:
+        num_products = len(list(app.order.iter_product_info()))
+        plural = "" if num_products == 1 else "s"
+        logger.info(f"Processing {num_products} product{plural}.")
     (chain,) = app.order.get_dict_entries("chain")
-
     if chain:
-        plural = "It" if num_products == 1 else "They"
-        logger.info(f"{plural} will be customized with the following parameters:")
+        plural = "" if num_products == 1 else "s"
+        logger.info(f"Product{plural} will be customized with the following parameters:")
         for line in yaml.dump(chain).splitlines():
             logger.info(f"   {line}")
-    if num_products >= 10 and not force:
-        user_in = input("Do you want to continue (Y/n)?")
-        if user_in.lower() == "n":
-            return
+
     logger.info(f"Using order: {app.order}")
     try:
         return app.run()
@@ -281,6 +296,12 @@ def safe_run(app: Any, force: bool = False) -> None:
         logger.info("Resume this order with the following command:")
         logger.info(f"$ eumdac order resume {app.order}")
         raise
+    except ProductError:
+        if collection:
+            app.datastore.check_collection_id(collection)
+            raise
+        else:
+            raise
     except Exception as e:
         logger.critical(f"Unexpected exception: {str(e)}")
         raise
@@ -292,7 +313,7 @@ def download(args: argparse.Namespace) -> None:
     datastore = get_datastore(args)
 
     if not args.collection or len(args.collection) > 1:
-        raise ValueError("Please provide a (single) collection")
+        raise ValueError("Please provide a (single) collection.")
 
     if args.time_range and (args.dtstart or args.dtend):
         raise ValueError("You can't combine --time-range and --start/--end.")
@@ -305,12 +326,29 @@ def download(args: argparse.Namespace) -> None:
     if args.product and (args.dtstart or args.dtend):
         raise ValueError("Please provide either products or a time-range!")
 
+    products: Collection[Product]
     if args.product:
-        products: Collection[Product] = [
-            datastore.get_product(_collection, pid) for pid in args.product
-        ]
+        products = [datastore.get_product(_collection, pid) for pid in args.product]
     else:
-        products = _search(args)
+        products = _search(args)[0]
+
+    products_count = len(products)
+
+    if products_count > 10000:
+        logger.info(f"Processing 10000 out of the total {products_count} products.")
+        products = itertools.islice(products, 10000)  # type: ignore
+        products_count = 10000
+        logger.error(
+            "Notice: EUMETSATs DataStore APIs allow a maximum of 10.000 items in a single request. If more than 10.000 items are needed, please split your requests."
+        )
+    else:
+        plural = "" if products_count == 1 else "s"
+        logger.info(f"Processing {products_count} product{plural}.")
+
+    if products_count >= 10 and not args.yes:
+        user_in = input("Do you want to continue (Y/n)? ")
+        if user_in.lower() == "n":
+            return
 
     try:
         query = products.search_query  # type: ignore
@@ -320,6 +358,7 @@ def download(args: argparse.Namespace) -> None:
     if args.tailor and args.chain:
         raise ValueError("Please provide either a tailor file or chain yaml string!")
     elif args.tailor or args.chain:
+        chain_config = ""
         if args.tailor:
             chain_config = args.tailor
         elif args.chain:
@@ -330,7 +369,7 @@ def download(args: argparse.Namespace) -> None:
                     chain = yaml.safe_load(file)
                 except:
                     logger.error("YAML file is corrupted. Please, check the YAML syntax.")
-                    exit()
+                    sys.exit()
         else:
             chain_config = chain_config.strip()
             if not chain_config.startswith("{"):
@@ -339,14 +378,14 @@ def download(args: argparse.Namespace) -> None:
                 chain = yaml.safe_load(chain_config)
             except:
                 logger.error("YAML string is corrupted. Please, check the YAML syntax.")
-                exit()
+                sys.exit()
         chain = Chain(**chain)
         order.initialize(chain, products, Path(args.output_dir), args.entry, query)
         app: Any = TailorApp(order, datastore, get_datatailor(args))
     else:
         order.initialize(None, products, Path(args.output_dir), args.entry, query)
         app = DownloadApp(order, datastore)
-    safe_run(app, args.yes)
+    safe_run(app, collection=args.collection[0], num_products=products_count, force=args.yes)
 
 
 def order(args: argparse.Namespace) -> None:
@@ -407,17 +446,17 @@ def order(args: argparse.Namespace) -> None:
                     logger.info(f"Order {order_name} successfully deleted.")
                 except:
                     logger.warning(f"Order {order_name} can't be deleted.")
-                exit()
+                sys.exit()
             else:
                 logger.info(f"Order {order_name} wasn't deleted.")
-                exit()
+                sys.exit()
         else:
             logger.info(f"Order {order_name} doesn't exist.")
-            exit()
+            sys.exit()
 
     if not order._order_file.is_file():
         logger.info(f"Order {order_name} doesn't exist.")
-        exit()
+        sys.exit()
 
     (typ,) = order.get_dict_entries("type")
     if typ == "download":
@@ -431,7 +470,7 @@ def order(args: argparse.Namespace) -> None:
 
     else:
         raise Exception(f"Unknown Order Type: {typ}")
-    safe_run(app, True)
+    safe_run(app, force=True)
 
 
 def get_datastore(args: argparse.Namespace, anonymous_allowed: bool = False) -> Any:
@@ -580,7 +619,7 @@ def tailor_post_job(args: argparse.Namespace) -> None:
                 chain = yaml.safe_load(file)
             except:
                 logger.error("YAML file is corrupted. Please, check the YAML syntax.")
-                exit()
+                sys.exit()
     else:
         chain_config = chain_config.strip()
         if not chain_config.startswith("{"):
@@ -589,7 +628,7 @@ def tailor_post_job(args: argparse.Namespace) -> None:
             chain = yaml.safe_load(chain_config)
         except:
             logger.error("YAML string is corrupted. Please, check the YAML syntax.")
-            exit()
+            sys.exit()
     chain = Chain(**chain)
     products = [datastore.get_product(collection_id, product_id) for product_id in product_ids]
     try:
@@ -776,11 +815,7 @@ def tailor_download(args: argparse.Namespace) -> None:
     url = token.urls.get("tailor", "customisations") + f"/{customisation_id}"
     response = requests.get(
         url,
-        headers={
-            "Authorization": "Bearer {}".format(token),
-            "referer": __documentation__,
-            "User-Agent": str(__title__ + "/" + __version__),
-        },
+        headers=dict(eumdac.common.headers, **{"Authorization": "Bearer {}".format(token)}),
     )
     if response.status_code == 200:
         results = response.json()[customisation_id]["output_products"]
@@ -798,11 +833,12 @@ def tailor_download(args: argparse.Namespace) -> None:
             url = token.urls.get("tailor", "download") + f"?path={result}"
             response = requests.get(
                 url,
-                headers={
-                    "Authorization": "Bearer {}".format(token),
-                    "referer": __documentation__,
-                    "User-Agent": str(__title__ + "/" + __version__),
-                },
+                headers=dict(
+                    eumdac.common.headers,
+                    **{
+                        "Authorization": "Bearer {}".format(token),
+                    },
+                ),
             )
             output = args.output_dir
             if response.status_code == 200:
@@ -861,8 +897,8 @@ def report_request_error(
     logger.error(f"{response.status_code} - {response.text}")
 
 
-class TailorHelpAction(argparse.Action):
-    """eumdac tailor -h entrypoint"""
+class HelpAction(argparse.Action):
+    """eumdac tailor/search/download/subscribe/order -h entrypoint"""
 
     def __call__(self, parser: argparse.ArgumentParser, *args: Any, **kwargs: Any) -> None:
         # Print the help if the command has 2 args,
@@ -945,7 +981,7 @@ def cli(command_line: Optional[Sequence[str]] = None) -> None:
         "-s",
         "--start",
         type=parse_isoformat,
-        help="UTC start date e.g. 2002-12-21T12:30:15",
+        help="UTC sensing start date e.g. 2002-12-21T12:30:15",
         metavar="YYYY-MM-DD[THH[:MM[:SS]]]",
         dest="dtstart",
     )
@@ -953,17 +989,10 @@ def cli(command_line: Optional[Sequence[str]] = None) -> None:
         "-e",
         "--end",
         type=parse_isoformat,
-        help="UTC end date e.g. 2002-12-21T12:30:15",
+        help="UTC sensing end date e.g. 2002-12-21T12:30:15",
         metavar="YYYY-MM-DD[THH[:MM[:SS]]]",
         dest="dtend",
     )
-    search_argument_parser.add_argument(
-        "--bbox",
-        nargs=4,
-        type=float,
-        metavar=("W", "S", "E", "N"),
-        help="filter by bounding box, the box is defined in EPSG:4326 decimal degrees",
-    )
     search_argument_parser.add_argument(
         "--time-range",
         nargs=2,
@@ -971,6 +1000,25 @@ def cli(command_line: Optional[Sequence[str]] = None) -> None:
         help="convenience search on UTC time range",
         metavar="YYYY-MM-DD[THH[:MM[:SS]]]",
     )
+    search_argument_parser.add_argument(
+        "--publication-after",
+        type=parse_isoformat,
+        help="filter by publication date, products ingested after this UTC date e.g. 2002-12-21T12:30:15",
+        metavar="YYYY-MM-DD[THH[:MM[:SS]]]",
+    )
+    search_argument_parser.add_argument(
+        "--publication-before",
+        type=parse_isoformat,
+        help="filter by publication date, products ingested before this UTC date e.g. 2002-12-21T12:30:15",
+        metavar="YYYY-MM-DD[THH[:MM[:SS]]]",
+    )
+    search_argument_parser.add_argument(
+        "--bbox",
+        nargs=4,
+        type=float,
+        metavar=("W", "S", "E", "N"),
+        help="filter by bounding box, the box is defined in EPSG:4326 decimal degrees",
+    )
     search_argument_parser.add_argument(
         "--geometry",
         help="filter by geometry, a custom geomtery in a EPSG:4326 decimal degrees.",
@@ -1029,6 +1077,9 @@ def cli(command_line: Optional[Sequence[str]] = None) -> None:
         epilog="example: %(prog)s -s 2020-03-01 -e 2020-03-15T12:15 -c EO:EUM:DAT:MSG:CLM",
         parents=[common_parser, search_argument_parser],
     )
+    parser_search.add_argument(
+        dest="print_help", nargs=0, action=HelpAction, help=argparse.SUPPRESS
+    )
     parser_search.set_defaults(func=search)
 
     parser_download = subparsers.add_parser(
@@ -1063,6 +1114,9 @@ def cli(command_line: Optional[Sequence[str]] = None) -> None:
         help="Chain YAML string for tailoring prior to downloading",
         metavar="CHAINYMLSTR",
     )
+    parser_download.add_argument(
+        dest="print_help", nargs=0, action=HelpAction, help=argparse.SUPPRESS
+    )
     parser_download.set_defaults(func=download)
 
     # subscribe parser
@@ -1072,7 +1126,7 @@ def cli(command_line: Optional[Sequence[str]] = None) -> None:
         parents=[common_parser],
     )
     parser_subscribe.add_argument(
-        dest="print_help", nargs=0, action=TailorHelpAction, help=argparse.SUPPRESS
+        dest="print_help", nargs=0, action=HelpAction, help=argparse.SUPPRESS
     )
 
     subscribe_subparsers = parser_subscribe.add_subparsers(dest="subscribe-command")
@@ -1126,7 +1180,7 @@ def cli(command_line: Optional[Sequence[str]] = None) -> None:
         parents=[common_parser],
     )
     parser_tailor.add_argument(
-        dest="print_help", nargs=0, action=TailorHelpAction, help=argparse.SUPPRESS
+        dest="print_help", nargs=0, action=HelpAction, help=argparse.SUPPRESS
     )
     tailor_subparsers = parser_tailor.add_subparsers(dest="tailor-command")
 
@@ -1237,9 +1291,7 @@ def cli(command_line: Optional[Sequence[str]] = None) -> None:
         help="manage orders",
         parents=[common_parser],
     )
-    parser_order.add_argument(
-        dest="print_help", nargs=0, action=TailorHelpAction, help=argparse.SUPPRESS
-    )
+    parser_order.add_argument(dest="print_help", nargs=0, action=HelpAction, help=argparse.SUPPRESS)
     order_subparsers = parser_order.add_subparsers(dest="order_command")
     order_parsers = {}
     for action in ["restart", "status", "resume", "delete"]:
