@@ -9,8 +9,6 @@ from xml.etree import ElementTree
 
 import requests
 
-from .__version__ import __documentation__, __title__, __version__  # noqa
-
 if TYPE_CHECKING:  # pragma: no cover
     import sys
     from typing import Any, Optional
@@ -24,6 +22,7 @@ if TYPE_CHECKING:  # pragma: no cover
     from eumdac.product import Product
 
 from eumdac.errors import EumdacError, eumdac_raise_for_status
+import eumdac.common
 
 
 class SearchResults:
@@ -114,10 +113,7 @@ class SearchResults:
                 url,
                 params=params,
                 auth=auth,
-                headers={
-                    "referer": __documentation__,
-                    "User-Agent": str(__title__ + "/" + __version__),
-                },
+                headers=eumdac.common.headers,
             )
         else:
             response = session.get(url, params=params, auth=auth)
@@ -193,10 +189,7 @@ class Collection:
             url,
             params={"format": "json"},
             auth=auth,
-            headers={
-                "referer": __documentation__,
-                "User-Agent": str(__title__ + "/" + __version__),
-            },
+            headers=eumdac.common.headers,
         )
         eumdac_raise_for_status(
             f"Could not get properties of {self._id}", response, CollectionError
@@ -240,11 +233,7 @@ class Collection:
         url = self.datastore.urls.get("tailor", "products")
         response = requests.get(
             url,
-            headers={
-                "Authorization": "Bearer {}".format(auth),
-                "referer": __documentation__,
-                "User-Agent": str(__title__ + "/" + __version__),
-            },
+            headers=dict(eumdac.common.headers, **{"Authorization": "Bearer {}".format(auth)}),
         )
         api_response = json.loads(response.text)
 
@@ -285,10 +274,7 @@ class Collection:
             response = requests.get(
                 url,
                 auth=auth,
-                headers={
-                    "referer": __documentation__,
-                    "User-Agent": str(__title__ + "/" + __version__),
-                },
+                headers=eumdac.common.headers,
             )
             eumdac_raise_for_status("Could not get search options", response, CollectionError)
             root = ElementTree.fromstring(response.text)
