@@ -11,8 +11,6 @@ import requests
 
 from eumdac.errors import EumdacError, eumdac_raise_for_status
 
-from .__version__ import __documentation__, __title__, __version__  # noqa
-
 if TYPE_CHECKING:  # pragma: no cover
     if sys.version_info < (3, 9):
         from typing import Generator, Iterable, Mapping, MutableMapping
@@ -23,6 +21,8 @@ if TYPE_CHECKING:  # pragma: no cover
 
     from eumdac.datatailor import DataTailor
 
+import eumdac.common
+
 
 class Customisation:
     _id: str
@@ -75,12 +75,7 @@ class Customisation:
                 "tailor", "customisation", vars={"customisation_id": self._id}
             )
             response = requests.get(
-                url,
-                auth=self.datatailor.token.auth,
-                headers={
-                    "referer": __documentation__,
-                    "User-Agent": str(__title__ + "/" + __version__),
-                },
+                url, auth=self.datatailor.token.auth, headers=eumdac.common.headers
             )
             eumdac_raise_for_status(
                 "Failed to get customisation", response, UnableToGetCustomisationError
@@ -140,10 +135,7 @@ class Customisation:
         response = requests.get(
             url,
             auth=self.datatailor.token.auth,
-            headers={
-                "referer": __documentation__,
-                "User-Agent": str(__title__ + "/" + __version__),
-            },
+            headers=eumdac.common.headers,
         )
         eumdac_raise_for_status("Failed to get customisation log", response, CustomisationError)
         return response.json()["log_content"]
@@ -162,10 +154,7 @@ class Customisation:
             auth=auth,
             params=params,
             stream=True,
-            headers={
-                "referer": __documentation__,
-                "User-Agent": str(__title__ + "/" + __version__),
-            },
+            headers=eumdac.common.headers,
         ) as response:
             eumdac_raise_for_status("Failed to get tailor download", response, CustomisationError)
             response.raw.name = output.split("/")[-1]
@@ -189,7 +178,7 @@ class Customisation:
             url = self.datatailor.urls.get("tailor", "delete")
             payload = {"uuids": [self._id]}
             auth = self.datatailor.token.auth
-            response = requests.patch(url, auth=auth, json=payload)
+            response = requests.patch(url, auth=auth, json=payload, headers=eumdac.common.headers)
             eumdac_raise_for_status("Failed to patch tailor delete", response, CustomisationError)
             self._deleted = True
 
@@ -199,7 +188,9 @@ class Customisation:
                 "tailor", "customisation", vars={"customisation_id": self._id}
             )
             auth = self.datatailor.token.auth
-            response = requests.put(url, json={"status": "killed"}, auth=auth)
+            response = requests.put(
+                url, json={"status": "killed"}, auth=auth, headers=eumdac.common.headers
+            )
             eumdac_raise_for_status(
                 "Failed to put tailor customisation", response, CustomisationError
             )
