@@ -21,6 +21,8 @@ def eumdac_raise_for_status(
         except json.decoder.JSONDecodeError:
             extra_info = {"text": response.text}
         extra_info.update({"url": url, "status": response.status_code})
+        if response.status_code > 500:
+            msg = msg + " (due to a server-side error)"
         exception = exc_cls(msg, extra_info)
         raise exception from exc
 
