@@ -5,8 +5,6 @@ from typing import TYPE_CHECKING
 
 import requests
 
-from .__version__ import __documentation__, __title__, __version__  # noqa
-
 if TYPE_CHECKING:  # pragma: no cover
     import sys
     from typing import Any
@@ -20,6 +18,7 @@ if TYPE_CHECKING:  # pragma: no cover
         from collections.abc import Mapping
 
 from eumdac.errors import EumdacError, eumdac_raise_for_status
+import eumdac.common
 
 
 class Subscription:
@@ -55,10 +54,7 @@ class Subscription:
             response = requests.get(
                 url,
                 auth=self.datastore.token.auth,
-                headers={
-                    "referer": __documentation__,
-                    "User-Agent": str(__title__ + "/" + __version__),
-                },
+                headers=eumdac.common.headers,
             )
             eumdac_raise_for_status(
                 "Update properties of subscription {self._id} failed", response, SubscriptionError
@@ -91,7 +87,9 @@ class Subscription:
         url = self.datastore.urls.get(
             "datastore", "subscription", vars={"subscription_id": self._id}
         )
-        response = requests.delete(url, auth=self.datastore.token.auth)
+        response = requests.delete(
+            url, auth=self.datastore.token.auth, headers=eumdac.common.headers
+        )
         eumdac_raise_for_status(
             "Delete subscription {self._id} failed", response, SubscriptionError
         )
