@@ -6,6 +6,7 @@ from dataclasses import asdict, dataclass
 from typing import TYPE_CHECKING
 
 import requests
+import eumdac.common
 
 if TYPE_CHECKING:  # pragma: no cover
     import sys
@@ -82,6 +83,7 @@ class Chain(AsDictMixin):
     resample_method: Optional[str] = None
     resample_resolution: Optional[list] = None  # type: ignore[type-arg]
     compression: Optional[dict] = None  # type: ignore[type-arg]
+    xrit_segments: Optional[list] = None  # type: ignore[type-arg]
 
     def __post_init__(self) -> None:
         for name, Model in self.__submodels.items():
@@ -142,6 +144,6 @@ class DataTailorCRUD:
         self._request("DELETE", url, auth=auth)
 
     def _request(self, method: str, url: str, **options: Any) -> requests.Response:
-        response = requests.request(method, url, **options)
+        response = requests.request(method, url, headers=eumdac.common.headers, **options)
         eumdac_raise_for_status(f"Request for {self.Model} failed.", response, EumdacError)
         return response
