@@ -11,6 +11,7 @@ from urllib.parse import quote as url_quote
 import requests
 from pkg_resources import resource_filename
 from requests.auth import AuthBase, HTTPBasicAuth
+import eumdac.common
 
 if TYPE_CHECKING:  # pragma: no cover
     from typing import Optional
@@ -115,7 +116,10 @@ class AccessToken:
         auth = HTTPBasicAuth(*self.credentials)
         now = time.time()
         response = requests.post(
-            self.urls.get("token", "token"), auth=auth, data={"grant_type": "client_credentials"}
+            self.urls.get("token", "token"),
+            auth=auth,
+            data={"grant_type": "client_credentials"},
+            headers=eumdac.common.headers,
         )
         response.raise_for_status()
         token_data = response.json()
